/*

   playstopper.c    piccola estensione al primo esempio del tutorial sulle 
   Vers. 0.0.2    librerie GTK+ che permette, tramite un fork, di
                    controllare un player audio non grafico interrompendo
                    la riproduzione del brano corrente cliccando sul bottone 
                    di interruzione.

                 Copyright (C)  1999-2004  Matteo Nastasi 

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

     CONTACT:    nastasi@alternativeoutput.it
                 matteo.nastasi@milug.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>

pid_t pid = 0;

gint checkchildren(gpointer   data)
{
    int stat;
    struct timespec ts = { 0, 300000000 }; 

    if (pid) {
        if (waitpid(pid,&stat,WNOHANG)) {
            gtk_main_quit();
            return (FALSE);
        }
    }

    nanosleep(&ts, NULL);

    return (TRUE);
}

gint delete_event( GtkWidget *widget,
                   GdkEvent  *event,
                   gpointer   data )
{
    /* If you return FALSE in the "delete_event" signal handler,
     * GTK will emit the "destroy" signal. Returning TRUE means
     * you don't want the window to be destroyed.
     * This is useful for popping up 'are you sure you want to quit?'
     * type dialogs. */

    g_print ("delete event occurred\n");

    /* Change TRUE to FALSE and the main window will be destroyed with
     * a "delete_event". */

    return(TRUE);
}

/* Another callback */
void destroy( GtkWidget *widget,
              gpointer   data )
{
    int stat;
  if (pid > 0) {
      kill(pid, SIGKILL);
      waitpid(pid,&stat,0);
  }
  gtk_main_quit();
}

extern char **environ;

#define OGGSTOPPER     0x00
#define WAVSTOPPER     0x01

static char *commpref[] = { "OGGSTOPPER=",
			    "WAVSTOPPER=" };

#define CONFIG_FILE    "/etc/playstopper.conf"

int main( int   argc,
          char *argv[] )
{
  int type;

  /* GtkWidget is the storage type for widgets */
  GtkWidget *window;
  GtkWidget *button;
  int ct;
  char bf[512], bf2[512], *p, *comm = NULL;
  FILE *fp;

  if (argc != 2) {
    fprintf(stderr,"invalid number of parameters\n");
    exit(1);
  }
  if ((p = strrchr(argv[0],'/')) != NULL) 
    p++;
  else 
    p = argv[0];
  
  if (!strcmp(p, "oggstopper"))
    type = OGGSTOPPER;
  else
    type = WAVSTOPPER;
  
  if (!(fp = fopen(CONFIG_FILE, "r"))) {
    fprintf(stderr,"cannot open config file [%s]\n",CONFIG_FILE);
    exit(1);
  }
  
  while (fgets(bf,512,fp)) {
    if (!strncasecmp(commpref[type],bf,strlen(commpref[type]))) {
      comm = &bf[strlen(commpref[type])];
      break;
    }
  }
  if (comm == NULL) {
    fprintf(stderr,"cannot found %s configuration line\n", commpref[type]);
    exit(2);
  }
  else {
    if ((p = strchr(comm,'\n')) != NULL)
      *p = '\0';
  }
  ct = 0;
  for (p = comm ; *p ; p++) {
    if (*p == '%')
      ct++;
  }
  // fprintf(stderr,"COMMAND [%s]\n",comm);
  if (ct != 1 || !strstr(comm,"%s")) {
    fprintf(stderr,"malformed [%s] configuration line\n", bf);
    exit(3);
  }

  sprintf(bf2,comm,argv[1]);
  
  // fprintf(stderr,"BF2[%s]\n", bf2);
  // exit(100);
  pid = fork();
  if (pid == 0) {
    char *argz[4];
    
    argz[0] = "sh";
    argz[1] = "-c";
    argz[2] = bf2;
    argz[3] = 0;
    
    execve("/bin/sh", argz, environ);
    exit(127);
  }
    /* This is called in all GTK applications. Arguments are parsed
     * from the command line and are returned to the application. */
    gtk_init(&argc, &argv);
    
    /* create a new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    
    /* When the window is given the "delete_event" signal (this is given
     * by the window manager, usually by the "close" option, or on the
     * titlebar), we ask it to call the delete_event () function
     * as defined above. The data passed to the callback
     * function is NULL and is ignored in the callback function. */
    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
                        GTK_SIGNAL_FUNC (delete_event), NULL);
    
    /* Here we connect the "destroy" event to a signal handler.  
     * This event occurs when we call gtk_widget_destroy() on the window,
     * or if we return FALSE in the "delete_event" callback. */
    gtk_signal_connect (GTK_OBJECT (window), "destroy",
                        GTK_SIGNAL_FUNC (destroy), NULL);
    
    /* Sets the border width of the window. */
    gtk_container_set_border_width (GTK_CONTAINER (window), 10);
    
    button = gtk_button_new_with_label (" Termina Ascolto ");
        
    /* This will cause the window to be destroyed by calling
     * gtk_widget_destroy(window) when "clicked".  Again, the destroy
     * signal could come from here, or the window manager. */
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                               GTK_SIGNAL_FUNC (gtk_widget_destroy),
                               GTK_OBJECT (window));
    
    /* This packs the button into the window (a gtk container). */
    gtk_container_add (GTK_CONTAINER (window), button);
    
    /* The final step is to display this newly created widget. */
    gtk_widget_show (button);
    
    /* and the window */
    gtk_widget_show (window);
    gtk_idle_add( checkchildren, NULL );
    /* All GTK applications must have a gtk_main(). Control ends here
     * and waits for an event to occur (like a key press or
     * mouse event). */
    gtk_main ();
    
    return(0);
}
/* example-end */
